@@ -1,6 +1,43 @@
-#' @import digest
+#' @import utils digest
 NULL
 
+# Reusable function for registering a set of methods with S3 manually. The
+# methods argument is a list of character vectors, each of which has the form
+# c(package, genname, class).
+registerMethods <- function(methods) {
+  lapply(methods, function(method) {
+    pkg <- method[[1]]
+    generic <- method[[2]]
+    class <- method[[3]]
+    func <- get(paste(generic, class, sep="."))
+    if (pkg %in% loadedNamespaces()) {
+      registerS3method(generic, class, func, envir = asNamespace(pkg))
+    }
+    setHook(
+      packageEvent(pkg, "onLoad"),
+      function(...) {
+        registerS3method(generic, class, func, envir = asNamespace(pkg))
+      }
+    )
+  })
+}
+
+.onLoad <- function(...) {
+  # htmltools provides methods for knitr::knit_print, but knitr isn't a Depends or
+  # Imports of htmltools, only an Enhances. Therefore, the NAMESPACE file has to
+  # declare it as an export, not an S3method. That means that R will only know to
+  # use our methods if htmltools is actually attached, i.e., you have to use
+  # library(htmltools) in a knitr document or else you'll get escaped HTML in your
+  # document. This code snippet manually registers our methods with S3 once both
+  # htmltools and knitr are loaded.
+  registerMethods(list(
+    # c(package, genname, class)
+    c("knitr", "knit_print", "html"),
+    c("knitr", "knit_print", "shiny.tag"),
+    c("knitr", "knit_print", "shiny.tag.list")
+  ))
+}
+
 depListToNamedDepList <- function(dependencies) {
   if (inherits(dependencies, "html_dependency"))
     dependencies <- list(dependencies)
@@ -176,6 +213,7 @@ normalizeText <- function(text) {
 
 }
 
+#' @name tag
 #' @rdname tag
 #' @export
 tagList <- function(...) {
